#!/bin/sh
#-------------------------------------------------------------------------------
# /var/install/config.d/xmdomu.sh - configuration generator script for xen
#
# Creation:     21.05.05 fw
# Last Update:  $Id: xmdomu.sh 20063 2009-06-11 22:22:41Z schlotze $
#
# Copyright (c) 2007 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#-------------------------------------------------------------------------------
. /lib/lsb/init-functions
. /var/install/include/xenlib

#exec 2> /tmp/xmdomu.sh-trace$$.log
#set -x

#
# domU configuration
#

# get current domain
currentDomU=`basename "$0" .sh`
name=`echo "$currentDomU" | sed 's#'$domUConfigPrefix$delimiter'##'`

rm -f $vmConfigPath/$vmConfigPrefix$name

. $configd/$currentDomU

if [ "$USE_CUSTOM_KERNEL" = 'no' ]
then
    KERNEL="/boot/vmlinuz-xen"
fi

if ! ls $fli4lPath/config.$name >/dev/null 2>&1 && [ "$IS_FLI4L" = "yes" ]
then
    if ! ls $fli4lPath >/dev/null 2>&1
    then
        # prepare the fli4l domU
        prepare_fli4l
    fi
    mkdir -p $fli4lPath/config.$name
    cp $fli4lPath/config/* $fli4lPath/config.$name
    mkdir -p $fli4lPath/help
    touch $fli4lPath/help/base.txt # XXX this is a work a round for the moment. Helpfile still missing!
fi

if ls $fli4lPath/config.$name >/dev/null 2>&1 && [ "$IS_FLI4L" = "yes" ]
then
    USE_CUSTOM_KERNEL="yes"
    KERNEL=$fli4lPath/config.$name/build/kernel
    rootfs=$fli4lPath/config.$name/build/rootfs.img
    KERNEL_PARAMS="$KERNEL_PARAMS nokbd"
    ROOT_DEVICE=/dev/tmpfs
else
    rootfs='/boot/initrd-xen.gz'
fi

if [ "$HVM" = "yes" ]
then
    KERNEL="/usr/lib/xen/boot/hvmloader"
fi

{
cat <<EOF
#  -*- mode: python; -*-
#============================================================================
# Python configuration setup for 'xm create'.
# This script sets the parameters used when a domain is created using 'xm create'.
# You use a separate script for each domain you want to create, or
# you can set the parameters for the domain on the xm command line.
#============================================================================

#----------------------------------------------------------------------------
# Kernel image file.
kernel = "$KERNEL"

# Optional ramdisk.
ramdisk = "$rootfs"

# Initial memory allocation (in megabytes) for the new domain.
memory = $MEMORY

# A name for your domain. All domains must have different names.
name = "$name"
#
#Which CPU to start domain on?
#cpu = -1   # leave to Xen to pick

EOF

if [ "$HVM" = "yes" ]
then
    echo "builder='hvm'"
    echo
fi

if [ "$HVM" = "yes" -a "$VNC_PASS" != "" ]
then
    echo "vnc=1"
    echo "vncviewer=0"
    echo "vncpasswd='$VNC_PASS'"
fi

if [ "$HVM" = "yes" ]
then
    echo "device_model = '/usr/lib/xen/bin/qemu-dm'"
fi

echo
echo -n "pci = ["

idx=1
while [ $idx -le $PCI_N ]
do
    eval id='$PCI_'$idx'_ID'

    echo -n " '$id'"

    if ! grep -q "$id" /sys/bus/pci/drivers/pciback/slots
    then
        if lspci -s "$id" >/dev/null
        then
            if find /sys/bus/pci/drivers -name "$id" >/dev/null
            then
                echo -n $id > "`find /sys/bus/pci/drivers -name "$id" | cut -d "/" -f-6`/unbind"
            fi 
            echo -n $id > /sys/bus/pci/drivers/pciback/new_slot
            echo -n $id > /sys/bus/pci/drivers/pciback/bind
        else
            log_failure_msg "The PCI ID PCI_${idx2}_ID='$id' desired by domU $name doesn't exist!" 
        fi
    fi    
    
    if [ $idx -le `expr $PCI_N - 1` ]
    then
        echo ", "
    fi

    let idx++
done

echo " ]"

irqs=""
ioports=""

idx=1
while [ $idx -le $LEGACY_N ]
do

    irqs=${irqs}""
    ioports=${ioports}"\""

    eval irq='$LEGACY_'${idx}'_IRQ'
    eval ioport='$LEGACY_'${idx}'_IOPORT'

    irqs=${irqs}${irq}
    ioports=${ioports}${ioport}

    if [ $idx -le `expr $LEGACY_N - 1` ]
    then
        irqs=${irqs}", "
        ioports=${ioports}"\", "
    else
        irqs=${irqs}""
        ioports=${ioports}"\""
    fi

    let idx++
done

echo
echo "irq = [ ${irqs} ]"
echo "ioports = [ ${ioports} ]"
echo

cat <<EOF

#----------------------------------------------------------------------------
# Define network interfaces.

# Optionally define mac and/or bridge for the network interfaces.
# Random MACs are assigned if not given.
EOF

echo -n "vif = ["

idx=1
while [ $idx -le $VIF_N ]
do
    eval mac='$VIF_'$idx'_MAC'
    eval bridge_id='$VIF_'$idx'_BRIDGE'

    eval netdev='$XEN_BRIDGE_'$bridge_id'_BIND_DEVS'

    bridge=$bridgeDevice`expr $bridge_id - 1`

    if [ "$mac" != "" ]
    then
        macstr="mac=$mac, "
    else
        macstr=""
    fi

    if [ "$HVM" = "yes" ]
    then
        typestr="type=ioemu, "
    else
        typestr=""
    fi

    echo -n " '$typestr${macstr}bridge=$bridge'"

    if [ $idx -le `expr $VIF_N - 1` ]
    then
        echo ", "
    fi

    bridge_idx=`expr $bridge_idx + 1`

    let idx++
done

cat <<EOF

       ]
#----------------------------------------------------------------------------
# Define the disk devices you want the domain to have access to, and
# what you want them accessible as.
# Each disk entry is of the form phy:UNAME,DEV,MODE
# where UNAME is the device, DEV is the device name the domain will see,
# and MODE is r for read-only, w for read-write.

EOF

echo -n "disk = ["

idx=1
while [ $idx -le $DISK_N ]
do
    eval type='$DISK_'$idx'_TYPE'
    eval uname='$DISK_'$idx'_UNAME'
    eval map='$DISK_'$idx'_MAP_TO'
    eval mode='$DISK_'$idx'_MODE'

    swapimage=$xenImagePath/$name.swap
    if [ "$xendRunning" = "yes" -a "$uname" = "autoswap" ]
    then
        checkIfVMIsUp $name
        if [ "$isUp" = "no" ]
        then
            check_swap
        fi
    elif [ "$uname" = "autoswap" ]
    then
        check_swap
    fi

    [ "$uname" = "autoswap" ] && uname=$swapimage

    if [ ! -e "$uname" -a "$type" = "file" ]
    then
        (
        mecho -error "The image"
        mecho -error "DISK_"$idx"_UNAME='$uname'"
        mecho -error "doesn't exist! Skipping..."
        )>`tty`
    else
        if [ "$HVM" = "yes" ]
        then
            ioemu="ioemu:"
        else
            ioemu=""
        fi

        echo -n " '$type:$uname,$ioemu$map,$mode'"

        if [ $idx -le `expr $DISK_N - 1` ]
        then
            echo -n ","
        fi
    fi

    idx=`expr $idx + 1`
done

cat <<EOF
 ]

#----------------------------------------------------------------------------
# Set the kernel command line for the new domain.
# You only need to define the IP parameters and hostname if the domain's
# IP config doesn't, e.g. in ifcfg-eth0 or via DHCP.
# You can use 'extra' to set the runlevel and custom environment
# variables used by custom rc scripts (e.g. VMID=, usr= ).

# Set if you want dhcp to allocate the IP address.
#dhcp="dhcp"
# Set netmask.
#netmask=
# Set default gateway.
#gateway=
# Set the hostname.
#hostname= "vm%d" % vmid

# Set root device.
root = "$ROOT_DEVICE"
boot = "$BOOT"

# Root device for nfs.
#root = "/dev/nfs"
# The nfs server.
#nfs_server = '169.254.1.0'
# Root directory on the nfs server.
#nfs_root   = '/full/path/to/root/directory'

# Sets runlevel 4.
extra = "$KERNEL_PARAMS"

#----------------------------------------------------------------------------
# Set according to whether you want the domain restarted when it exits.
# The default is 'onreboot', which restarts the domain when it shuts down
# with exit code reboot.
# Other values are 'always', and 'never'.

#restart = 'onreboot'

#============================================================================
EOF
    } >$vmConfigPath/$vmConfigPrefix$name

# XXX: api handling
# check if domain is running.
checkIfVMIsUp $name # returns $isUp=yes/no
# when domain is up, create file indicating
# pending changes
if [ "$isUp" = "yes" ]
then
    mkdir -p /etc/xen/pending-changes/
    touch /etc/xen/pending-changes/$name
# if domain is down write changes directly
# into xen repository
else
    xm del $name >/dev/null 2>&1
    registrate_domain $name
    rm -f /etc/xen/pending-changes/$name
    rmdir /etc/xen/pending-changes >/dev/null 2>&1
fi

exit 0
